"""
Unit test file.
"""

import unittest

from fastled.print_filter import BuildArtifact

_BUILD_CONFIG_SAMPLE = """
6.01 ccache em++ -o .pio/build/wasm/libbb9/fastled/FastLED.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/FastLED.cpp
6.02 ccache em++ -o .pio/build/wasm/libbb9/fastled/bitswap.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/bitswap.cpp
6.04 ccache em++ -o .pio/build/wasm/libbb9/fastled/cled_controller.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/cled_controller.cpp
6.05 ccache em++ -o .pio/build/wasm/libbb9/fastled/colorpalettes.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/colorpalettes.cpp
6.07 ccache em++ -o .pio/build/wasm/libbb9/fastled/crgb.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/crgb.cpp   
6.07 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/allocator.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/allocator.cpp
6.08 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/audio.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/audio.cpp
6.09 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/bilinear_expansion.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/bilinear_expansion.cpp
6.10 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/blur.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/blur.cpp
6.10 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/bytestreammemory.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler 
src/fl/bytestreammemory.cpp
6.12 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/colorutils.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/colorutils.cpp
6.12 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/downscale.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/downscale.cpp
6.13 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/engine_events.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/engine_events.cpp
6.14 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/fft.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/fft.cpp
6.16 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/fft_impl.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline 
-DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/fft_impl.cpp
6.17 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/file_system.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/file_system.cpp
6.17 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/fill.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/fill.cpp
6.19 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/five_bit_hd_gamma.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/five_bit_hd_gamma.cpp
6.19 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/gradient.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline 
-DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/gradient.cpp
6.20 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/json.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/json.cpp
6.20 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/leds.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/leds.cpp
6.21 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/line_simplification.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/line_simplification.cpp
6.22 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/noise_woryley.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/noise_woryley.cpp
6.23 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/ptr.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/ptr.cpp
6.24 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/raster_sparse.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/raster_sparse.cpp
6.24 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/rectangular_draw_buffer.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/rectangular_draw_buffer.cpp
6.26 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/screenmap.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/screenmap.cpp
6.26 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/sin32.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/sin32.cpp
6.27 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/str.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/str.cpp
6.27 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/strstream.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/strstream.cpp
6.28 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/stub_main.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/stub_main.cpp
6.29 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/tile2x2.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/tile2x2.cpp
6.29 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/time_alpha.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/time_alpha.cpp
6.30 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/transform.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/transform.cpp
6.31 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/type_traits.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/type_traits.cpp
6.32 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/ui.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/ui.cpp 
6.33 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/wave_simulation.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type 
-I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/wave_simulation.cpp
6.34 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/wave_simulation_real.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined 
-fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/wave_simulation_real.cpp
6.34 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/xmap.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/xmap.cpp
6.35 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/xymap.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/xymap.cpp
6.36 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/xypath.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/xypath.cpp
6.36 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/xypath_impls.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/xypath_impls.cpp
6.37 ccache em++ -o .pio/build/wasm/libbb9/fastled/fl/xypath_renderer.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type 
-I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fl/xypath_renderer.cpp
6.39 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/2d/blend.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline 
-DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/2d/blend.cpp
6.40 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/2d/noisepalette.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type 
-I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/2d/noisepalette.cpp
6.41 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/2d/scale_up.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/2d/scale_up.cpp
6.42 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/2d/wave.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/2d/wave.cpp
6.42 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/frame.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/frame.cpp
6.43 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/fx_engine.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/fx_engine.cpp
6.44 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/time.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/time.cpp
6.45 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/video.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/video.cpp
6.46 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/video/frame_interpolator.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/video/frame_interpolator.cpp
6.46 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/video/frame_tracker.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/video/frame_tracker.cpp
6.47 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/video/pixel_stream.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/fx/video/pixel_stream.cpp
6.48 ccache em++ -o .pio/build/wasm/libbb9/fastled/fx/video/video_impl.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler 
src/fx/video/video_impl.cpp
6.49 ccache em++ -o .pio/build/wasm/libbb9/fastled/hsv2rgb.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/hsv2rgb.cpp
6.50 ccache em++ -o .pio/build/wasm/libbb9/fastled/lib8tion.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/lib8tion.cpp
6.51 ccache em++ -o .pio/build/wasm/libbb9/fastled/noise.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/noise.cpp 
6.52 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms.cpp
6.53 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/arm/k20/clockless_objectfled.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/arm/k20/clockless_objectfled.cpp
6.54 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/clockless_i2s_esp32s3.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/clockless_i2s_esp32s3.cpp
6.55 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/i2s/i2s_esp32dev.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/i2s/i2s_esp32dev.cpp
6.56 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/rmt_4/idf4_rmt.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/rmt_4/idf4_rmt.cpp
6.57 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/rmt_4/idf4_rmt_impl.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/rmt_4/idf4_rmt_impl.cpp
6.59 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/rmt_5/idf5_rmt.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/rmt_5/idf5_rmt.cpp
6.60 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/rmt_5/strip_rmt.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/rmt_5/strip_rmt.cpp
6.61 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/esp/32/spi_ws2812/strip_spi.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/esp/32/spi_ws2812/strip_spi.cpp
6.62 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/active_strip_data.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/active_strip_data.cpp
6.63 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/compiler/Arduino.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/compiler/Arduino.cpp
6.64 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/engine_listener.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/engine_listener.cpp
6.65 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/fs_wasm.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/fs_wasm.cpp
6.66 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/js.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/js.cpp
6.66 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/timer.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/timer.cpp
6.67 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/audio.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined 
-fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/audio.cpp
6.69 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/button.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/button.cpp
6.70 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/checkbox.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/checkbox.cpp
6.71 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/description.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/description.cpp
6.72 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/number_field.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/number_field.cpp
6.73 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/slider.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/slider.cpp
6.74 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/title.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined 
-fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/title.cpp
6.74 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/ui_internal.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/ui_internal.cpp
6.76 ccache em++ -o .pio/build/wasm/libbb9/fastled/platforms/wasm/ui/ui_manager.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/platforms/wasm/ui/ui_manager.cpp
6.77 ccache em++ -o .pio/build/wasm/libbb9/fastled/power_mgt.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/power_mgt.cpp
6.78 ccache em++ -o .pio/build/wasm/libbb9/fastled/rgbw.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/rgbw.cpp   
6.79 ccache em++ -o .pio/build/wasm/libbb9/fastled/sensors/digital_pin.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler 
src/sensors/digital_pin.cpp
6.80 ccache em++ -o .pio/build/wasm/libbb9/fastled/sensors/pir.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline 
-DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/sensors/pir.cpp
6.81 ccache em++ -o .pio/build/wasm/libbb9/fastled/simplex.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/simplex.cpp
6.83 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/cq_kernel/cq_kernel.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/cq_kernel/cq_kernel.c
6.84 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/cq_kernel/kiss_fft.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler 
src/third_party/cq_kernel/kiss_fft.c
6.84 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/cq_kernel/kiss_fftr.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/cq_kernel/kiss_fftr.c
6.85 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/espressif/led_strip/src/led_strip_api.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/espressif/led_strip/src/led_strip_api.c
6.86 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/espressif/led_strip/src/led_strip_rmt_dev.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/espressif/led_strip/src/led_strip_rmt_dev.c
6.87 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/espressif/led_strip/src/led_strip_rmt_encoder.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/espressif/led_strip/src/led_strip_rmt_encoder.c
6.88 ccache emcc -o .pio/build/wasm/libbb9/fastled/third_party/espressif/led_strip/src/led_strip_spi_dev.o -c -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/espressif/led_strip/src/led_strip_spi_dev.c
6.88 ccache em++ -o .pio/build/wasm/libbb9/fastled/third_party/object_fled/src/OjectFLED.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/third_party/object_fled/src/OjectFLED.cpp
6.89 ccache em++ -o .pio/build/wasm/libbb9/fastled/transpose8x1_noinline.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/transpose8x1_noinline.cpp
6.90 ccache em++ -o .pio/build/wasm/libbb9/fastled/wiring.o -c -std=c++17 -Werror=bad-function-cast -Werror=cast-function-type -I/js/src/platforms/wasm/compiler -g -O0 -gsource-map=inline -ffile-prefix-map=/=drawfsource/ -fsanitize=address -fsanitize=undefined -fno-inline -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc/platforms/wasm/compiler src/wiring.cpp
6.92 ccache em++ -o .pio/build/wasm/src/XYPath.ino.o -c -std=c++17 -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc -Isrc/platforms/wasm/compiler src/XYPath.ino.cpp
6.94 ccache em++ -o .pio/build/wasm/src/src/wave.o -c -std=c++17 -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc -Isrc/platforms/wasm/compiler src/src/wave.cpp
6.94 ccache em++ -o .pio/build/wasm/src/src/xypaths.o -c -std=c++17 -DPLATFORMIO=60117 -DFASTLED_NO_PINMAP -DHAS_HARDWARE_PIN_SUPPORT -DFASTLED_FORCE_SOFTWARE_SPI -Isrc -Isrc -Isrc/platforms/wasm/compiler src/src/xypaths.cpp
"""


class Filter:
    def __init__(self):
        self._prev_broken_line: str | None = None
        self._last_build_artifact: BuildArtifact | None = None

    def filter(self, line: str) -> str | None:
        if not line:
            return None
        maybe_prev_broken_line: str | None = self._prev_broken_line
        self._prev_broken_line = None
        # if the line starts with a digit, it's a build artifact
        ba: BuildArtifact | None = BuildArtifact.parse(line)

        if ba is not None:  # Successfully parsed a build artifact.
            # if we have a build artifact, we need to check if it's the same as the last one
            if (
                self._last_build_artifact is not None
                and ba.hash == self._last_build_artifact.hash
            ):
                # No change in build settings.
                out = ba.brief()
                self._last_build_artifact = ba
                return out
            if self._last_build_artifact is None:
                self._last_build_artifact = ba
                return ba.begin_flags() + "\n" + ba.brief()
            # we have a new build artifact, so we need to print out the build settings
            # and the line
            out = self._last_build_artifact.end_flags() + "\n\n\n"
            out += ba.begin_flags()
            out += "\n" + ba.brief()
            self._last_build_artifact = ba
            return out

        # We couldn't parse this line so store it in stead.
        if maybe_prev_broken_line is None:
            self._prev_broken_line = line
            return None
        # we have a previous line, so we can try to merge them because this typically happens
        # for line breaks.
        new_line = maybe_prev_broken_line + line
        ba = BuildArtifact.parse(new_line)
        if ba is None:
            return None
        # we have a build artifact, so we can use it.

        if self._last_build_artifact and (ba.hash == self._last_build_artifact.hash):
            # No change in build settings.
            out = ba.brief()
            return out
        out = ""
        # different build settings so we need to print out the new build settings and then the line
        if self._last_build_artifact is not None:
            out += "\n" + self._last_build_artifact.end_flags() + "\n\n"
        out += ba.begin_flags()
        out += ba.brief() + "\n"
        self._last_build_artifact = ba
        return out

    def end(self) -> str:
        """Returns the end of the build artifact."""
        if self._last_build_artifact is None:
            return ""
        # we have a build artifact, so we can use it.
        out = self._last_build_artifact.end_flags()
        self._last_build_artifact = None
        return out


class ExperimentalPrintFitlerCppTester(unittest.TestCase):
    """Main tester class."""

    def test_build_artifact_parsing(self) -> None:
        """Tests that a project can be filtered"""
        # Test the PrintFilter class
        lines = _BUILD_CONFIG_SAMPLE.splitlines()
        lines = [line.strip() for line in lines if line.strip()]
        filter: Filter = Filter()
        for line in lines:
            str_or_none = filter.filter(line)
            if str_or_none is not None:
                print(str_or_none)
        print(filter.end())


if __name__ == "__main__":
    unittest.main()
